% 2D FBP Algorithm Example
% PET Basics 2018 Course, 3.4.2018
% 
% Modified for Octave from MATLAB on 9.5.2018
% Jarmo Teuho 2018
close all;clear all

%% Noiseless case

% Create a Shepp-Logan phantom
IM_ref = phantom(128); 
phi = 0:(180/128):(180-1/(128+1)); % all angles

% Forward project phantom to create simulated, measured projection data
[p_measured,xp] = radon(IM_ref,phi); 
	
% Do backprojection without applying the Ramp filter
f_no_ramp=iradon(p_measured,phi,'Linear','None' ,128);

% Do backprojection and apply Ramp filter
f_ramp=iradon(p_measured,phi,'Linear','Ram-Lak');

% Visualize result
figure;
subplot(2,2,1),imshow(IM_ref,[]),title('Reference object'),colorbar
subplot(2,2,2),imshow(p_measured',[]),title('Sinogram'),colorbar
subplot(2,2,4),imshow(f_no_ramp,[]),title('Backprojection (no ramp filter)'),colorbar
subplot(2,2,3),imshow(f_ramp,[]),title('FBP (ramp filter)'),colorbar
set(gcf,'Position',[67 70 916 722])

%% Noisy case

% Add noise to the projection data
p_measured_noisy = poissrnd(p_measured);

% Do backprojection without applying the Ramp filter
f_noisy_ramp=iradon(p_measured_noisy,phi,'Linear','Ram-Lak');

% Do backprojection and apply a different filter function
f_noisy_hann=iradon(p_measured_noisy,phi,'Linear','Hann');

% Visualize result
figure;
subplot(2,2,1),imshow(IM_ref,[]),title('Reference object'),colorbar
subplot(2,2,2),imshow(p_measured_noisy',[]),title('Sinogram with noise'),colorbar
subplot(2,2,4),imshow(f_noisy_ramp,[]),title('FBP (Ramp filter)'),colorbar
subplot(2,2,3),imshow(f_noisy_hann,[]),title('FBP (Hann filter)'),colorbar
set(gcf,'Position',[67 70 916 722])

%% Missing data in the sinogram

% Add gaps to the projection data
p_measured_gaps = p_measured;
p_measured_gaps(:,55:65)=0;

% Do backprojection without applying the Ramp filter
f_noisy_ramp=iradon(p_measured_gaps,phi,'Linear','Ram-Lak');

% Do backprojection and apply a different filter
f_noisy_hann=iradon(p_measured_gaps,phi,'Linear','Hann');

% Visualize result
figure;
subplot(2,2,1),imshow(IM_ref,[]),title('Reference object'),colorbar
subplot(2,2,2),imshow(p_measured_gaps',[]),title('Sinogram with gaps'),colorbar
subplot(2,2,4),imshow(f_noisy_ramp,[]),title('FBP (Ramp filter)'),colorbar
subplot(2,2,3),imshow(f_noisy_hann,[]),title('FBP (Hann filter)'),colorbar
set(gcf,'Position',[67 70 916 722])
