% 2D MLEM Algorithm Example
% Original: PET Basics 2018 Course, 3.4.2018
% 
% Modified for Octave from MATLAB on 9.5.2018
% Jarmo Teuho 2018
close all;clear all

% Define the number of iterations
ITER=8;

%% Noiseless case

% Create a Shepp-Logan phantom
IM_ref = phantom(128); 
phi = 0:(180/128):(180-1/(128+1)); % all angles

% Forward project phantom to create simulated, measured projection data
[p_measured,xp] = radon(IM_ref,phi); 

% Create system model H and initial image f
s = iradon(ones(128,128),phi,'Linear','None',1,128); % system model
f = ones(128,128); % initial image

% Run MLEM
f=mlem(ITER,p_measured,f,s,phi);

figure;
subplot(1,3,1),imshow(IM_ref,[]),title('Reference'),colorbar
subplot(1,3,2),imshow(f,[]),title('Noiseless phantom'),colorbar

%% Noisy case

% Add noise to the projection data
p_measured_noisy = poissrnd(p_measured);

% Create system model H and initial image f
s = iradon(ones(128,128),phi,'Linear','None',1,128); % system model
f = ones(128,128); % initial image

% Run MLEM
f_noisy=mlem(ITER,p_measured_noisy,f,s,phi);

subplot(1,3,3),imshow(f_noisy,[]),title('Noisy phantom'),colorbar

%% Missing Data

% Add gaps to the projection data
p_measured_gaps = p_measured;
p_measured_gaps(:,55:65)=0;

% Create system model H and initial image f
s = iradon(ones(128,128),phi,'Linear','None',1,128); % system model
f = ones(128,128); % initial image

% Run MLEM
f_gaps=mlem(ITER,p_measured_gaps,f,s,phi);

% Visualize result
figure;
subplot(2,2,1),imshow(IM_ref,[]),title('Reference object'),colorbar
subplot(2,2,2),imshow(p_measured_gaps',[]),title('Sinogram with gaps'),colorbar
subplot(2,2,4),imshow(p_measured',[]),title('Original sinogram'),colorbar
subplot(2,2,3),imshow(f_gaps,[]),title('MLEM Reconstructed'),colorbar
set(gcf,'Position',[67 70 916 722])
