% Example of Point Source Recovery, 2D FBP and MLEM Algorithms
% Original: UTU Signal and Image Processing Course, 12.5.2022
% 
% Modified for Octave from MATLAB
% Jarmo Teuho 2022
close all;clear all

%% Noiseless vs noisy case

% Create a point source
IM_ref = zeros(128,128);
IM_ref(64:66,64:66)=1;

% Blur with Gaussian kernel
IM_ref=imsmooth(IM_ref,'Gaussian',1.5); % OCTAVE

phi = 0:(180/128):(180-1/(128+1)); % all angles

% Forward project to create simulated, measured projection data
[p_measured,xp] = radon(IM_ref,phi); 

% Add noise to the projection data
p_measured_noisy = poissrnd(p_measured);
	
% Do backprojection and apply Ramp filter
f_FBP_ramp=iradon(p_measured,phi,'Linear','Ram-Lak'); % noisless case
%f_FBP_ramp=iradon(p_measured_noisy,phi,'Linear','Ram-Lak'); % noisy case

% Change from 2 to 8 (or large number)
ITER=2;

% Do MLEM reconstruction 
% Create system model H and initial image f
s = iradon(ones(128,128),phi,'Linear','None',1,128); % system model
f = ones(128,128); % initial image

% Run MLEM
f_MLEM=mlem(ITER,p_measured,f,s,phi); % noisless case
% f_MLEM=mlem(ITER,p_measured_noisy,f,s,phi); % noisy case

% Visualize result
figure;
subplot(2,2,1),imshow(IM_ref,[]),title('Reference object'),colorbar
subplot(2,2,2),imshow(p_measured',[]),title('Sinogram'),colorbar
subplot(2,2,4),imshow(f_MLEM,[]),title('MLEM '),colorbar
subplot(2,2,3),imshow(f_FBP_ramp,[]),title('FBP'),colorbar
set(gcf,'Position',[67 70 916 722])

%% % Measure reconstructed image FWHM and compare to reference
FWHM_ref=calculate_line_profile(IM_ref,65,61:69); % MATLAB indexing 61:69,61:69
FWHM_FBP=calculate_line_profile(f_FBP_ramp,65,61:69)
FWHM_MLEM=calculate_line_profile(f_MLEM,65,61:69)

figure;plot(FWHM_ref,'--')
hold on
plot(FWHM_FBP,'*-')
hold on
plot(FWHM_MLEM,'o-')
legend('Ref','FBP','MLEM')



